local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 38661 $"):match("%d+"))

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

ThreatLib_funcs[#ThreatLib_funcs+1] = function()

local _G = _G
local tonumber = _G.tonumber
local pairs = _G.pairs

local UnitName = _G.UnitName
local UnitAffectingCombat = _G.UnitAffectingCombat
local GetPetActionInfo = _G.GetPetActionInfo

local AceLibrary = _G.AceLibrary

local ThreatLib = _G.ThreatLib

-- local _, c = _G.UnitClass("player")
-- We'll let it load for everyone, because sometimes non-hunter/locks get pets.
-- if c ~= "HUNTER" and c ~= "WARLOCK" then return end 
local c = "PET"
local new, del, newHash, newSet = ThreatLib.new, ThreatLib.del, ThreatLib.newHash, ThreatLib.newSet

local BS = AceLibrary("Babble-Spell-2.2")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local Pet = ThreatLib:GetModule("ClassCore"):NewModule(c)

-- Most of theses data come from KTM's pet module
local skillData = {
	-- Scaling skills
	[BS["Growl"]] = {
		rankLevel    = { 1, 10,  20,  30,  40,  50,  60,  70},
		rankThreat   = {50, 65, 110, 170, 240, 320, 415, 664},
		apBaseBonus  = 1235.6,
		apLevelMalus = 28.14,
		apFactor     = 5.7,
	},	
	[BS["Anguish"]] = {
		rankLevel    = { 50,  60,  69},
		rankThreat   = {300, 395, 632},
		apBaseBonus  = 109,
		apLevelMalus = 0,
		apFactor     = 0.698,
	},
	[BS["Torment"]] = {
		rankLevel    = {10, 20,  30,  40,  50,  60,  70},
		rankThreat   = {45, 75, 125, 215, 300, 395, 632},
		apBaseBonus  = 123,
		apLevelMalus = 0,
		apFactor     = 0.385,
	},
	[BS["Suffering"]] = {
	  rankLevel    = { 24,  36,  48,  60,  63,  69},
		rankThreat   = {150, 300, 450, 600, 645, 885},
		apBaseBonus  = 124,
		apLevelMalus = 0,
		apFactor     = 0.547,	
	},
	
	-- Unscaling skills
	[BS["Cower"]] = {
		rankThreat = {-30, -55,  -85, -125, -175, -225, -360},	
	},
	[BS["Cleave"]] = {
		rankThreat = {0, 0, 0, 0, 100, 130},
	},
	[BS["Soothing Kiss"]] = {
		rankThreat = {-45, -75, -127, -165, -275},
	},
}

function Pet:ClassInit()
	-- CastHandlers
	self.petName = UnitName("pet")
	self.isPetModule = true
	self.unitType = "pet"
	
	local playerClass = select(2, UnitClass("player"))
	self.petScaling = (playerClass == "HUNTER") or (playerClass == "WARLOCK")
	
	local function castHandler(self, rank, name) self:AddSkillThreat(name, rank) end
	for name in pairs(skillData) do
		self.CastHandlers[name] = castHandler
	end
	
	self:RegisterEvent("LOCALPLAYER_PET_RENAMED")
	self:RegisterEvent("UNIT_NAME_UPDATE")
	self:RegisterEvent("UNIT_PET")
	self.skillRanks = {}
	self:ScanPetSkillRanks()
	self:RegisterEvent("PET_BAR_UPDATE", "ScanPetSkillRanks")	
end

function Pet:ScanPetSkillRanks()
	for i = 1,10 do
		local name, rank = GetPetActionInfo(i)
		if skillData[name] then
			self.skillRanks[name] = rank
		end
	end
	self:UpdateScalingData()
end

function Pet:UpdateScalingData()
	local petLevel = UnitExists("pet") and UnitLevel("pet")
	if not self.petScaling or not petLevel or (self.petLevel and self.petLevel == petLevel) then
		return
	end
	self.petLevel = petLevel
	
	-- Clean skill data
	for name, skill in pairs(skillData) do
		
		if skill.scaledRankThreat then
			skill.scaledRankThreat = del(skill.scaledRankThreat)
		end
		
		if self.skillRanks[name] and skill.apFactor then
			-- Calculate scaling
			
			-- AP threshold
			skill.scaledAPThreshold = math.max(skill.apLevelMalus * petLevel - skill.apBaseBonus, 0)
			
			-- Find best available rank
			local levels = skill.rankLevel
			local bestRank = #levels
			for rank = 2, #levels-1 do 
				if petLevel < levels[rank] then
					bestRank = rank
					break
				end
			end

			-- Calculate scaled threat for each available rank
			skill.scaledRankThreat = new()
			for rank = 1, bestRank do
				local nextRank = math.min(rank+1, #levels)
				local minLevel, maxLevel = levels[rank], levels[nextRank]
				local minThreat, maxThreat = skill.rankThreat[rank], skill.rankThreat[nextRank]
				if petLevel <= minLevel then
					skill.scaledRankThreat[rank] = minThreat
				elseif petLevel >= maxLevel then
					skill.scaledRankThreat[rank] = maxThreat
				else
					local levelFactor = (petLevel - minLevel) / (maxLevel - minLevel)
					skill.scaledRankThreat[rank] = minThreat + levelFactor * (maxThreat - minThreat)
				end
			end
			
		else
			skill.scaledAPThreshold = nil			
		end
	end
end

function Pet:AddSkillThreat(name, rank)
	rank = rank or self.skillRanks[name]
	local ranknum = tonumber(rank:match("%d+"))
	local skill = skillData[name]
	local threat
	if not skill.scaledAPThreshold then
		-- Unscaling skill
		threat = skill.rankThreat[ranknum]
	else
		-- Scaling skill
		threat = skill.scaledRankThreat[ranknum]
		
		if self.petLevel and self.petLevel > 59 then
			local baseAP, posAPBuff, negAPBuff = UnitAttackPower("pet");	
			local petAP = baseAP + posAPBuff + negAPBuff;
			if petAP > skill.scaledAPThreshold then
				threat = threat + skill.apFactor * (petAP - skill.scaledAPThreshold)
			end
		end
		
	end
	if not threat then return end
	self:AddTargetThreat(math.floor(threat+0.5) * self:threatMods())	
end

function Pet:LOCALPLAYER_PET_RENAMED()
	self.petName = UnitName("pet")
end

local function reinitme(self)
	ThreatLib:GetModule("ClassCore"):ToggleModuleActive(self, false)
	ThreatLib:GetModule("ClassCore"):ToggleModuleActive(self, true)
end

function Pet:UNIT_PET(arg1)
	if arg1 == "player" then
		self.petName = UnitName("pet")
		if self.petName then
			self:ScheduleEvent("ThreatReInitPetModule", reinitme, 0.5, self)
			ThreatLib:PARTY_MEMBERS_CHANGED()
			self:UpdateScalingData()
			ThreatLib:PLAYER_ENTERING_WORLD()
		end		
	end
end

function Pet:UNIT_NAME_UPDATE(arg1)
	if arg1 == "pet" then	
		self.petName = UnitName("pet")
		if self.petName then
			self:ScheduleEvent("ThreatReInitPetModule", reinitme, 0.5, self)
			ThreatLib:PARTY_MEMBERS_CHANGED()
			self:UpdateScalingData()
		end		
	end
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	Pet = ThreatLib:GetModule("ClassCore"):GetModule(c)
end)

end