-----------------------------------------------------------------------------
--
-- data
--   \deaths
--     \<deathName>
--       - count
--       \deathTimes (Array)
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDeaths")
L:RegisterTranslations("enUS", function() return {
	["Deaths List"] = true,
	["Deaths"] = true,
	["Death Times"] = true,
	["Kill List"] = true,
	["Kill Times"] = true,
} end)

-----------------------------------------------------------------------------

local AssessmentDeaths = Assessment:NewModule("Deaths", "AceEvent-2.0", "Parser-3.0")
AssessmentDeaths.db = Assessment:AcquireDBNamespace("Deaths")
Assessment:RegisterDefaults("Deaths", "profile", {
	enable = true
})

-----------------------------------------------------------------------------

Assessment.TagData.deaths =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local deaths = fightData.deaths
				if not deaths then return 0 end
				
				if not deaths[data.key] then return 0 end
				return deaths[data.key].count
			 end,
	data = "deaths",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.DeathsList =
{
	method = function(fightData)
				if not fightData.deaths then return function() return nil end end
				return pairs(fightData.deaths)
			 end,
	data = "deaths",
	description = L["Deaths List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.DeathTimes =
{
	titleTag			= "[windowDisplaySetParameter(2)]",
	listType			= "EnumerateTable", 
	barValueTag			= "0", 
	filterTag			= "1",
	fightIndex			= 0,
	text				= { count = 2, { tag= "Death [rank]" }, { tag = "[value:formatTime]" } },
	description			= L["Death Times"],
	selectable			= false
}

Assessment.DefaultDisplaySets.Deaths =
{
	titleTag			= "Deaths",
	listType			= "GroupMembers", 
	barValueTag			= "deaths", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 0,
	description			= L["Deaths"],
	onClickType			= "PUSH_DISPLAY_SET",
	onClickDisplaySet	= L["Death Times"],
	onClickData			= "deaths,[key],deathTimes",
	favorite			= false,
}

Assessment.DefaultDisplaySets.KillTimes =
{
	titleTag			= "[windowDisplaySetParameter(2)]",
	listType			= "EnumerateTable", 
	barValueTag			= "0", 
	filterTag			= "1",
	fightIndex			= 0,
	text				= { count = 2, { tag= "Kill [rank]" }, { tag = "[value:formatTime]" } },
	description			= L["Kill Times"],
	selectable			= false
}

Assessment.DefaultDisplaySets.KillList =
{
	titleTag			= "Kill List",
	listType			= "DeathsList", 
	barValueTag			= "deaths", 
	filterTag			= "~isGroupMember",
	fightIndex			= 0,
	description			= L["Kill List"],
	onClickType			= "PUSH_DISPLAY_SET",
	onClickDisplaySet	= L["Kill Times"],
	onClickData			= "deaths,[key],deathTimes"
}

-----------------------------------------------------------------------------

function AssessmentDeaths:OnInitialize()
end

function AssessmentDeaths:OnEnable()
	self:RegisterParserEvent({ eventType = "Death" }, "OnParserDeathEvent")
end

function AssessmentDeaths:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentDeaths:OnParserDeathEvent(eventData)
	-- Ignore the slain messages.. to avoid double booking
	if eventData.sourceName then return end

	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.deaths = GetTime()
		local deaths = data.deaths
		if not deaths then
			deaths = self:NewTable()
			data.deaths = deaths
		end

		local recipientDeath = deaths[eventData.recipientName]
		if recipientDeath then
			recipientDeath.count = recipientDeath.count + 1
		else
			recipientDeath = self:NewTable()
			recipientDeath.count = 1
			recipientDeath.deathTimes = self:NewTable()
			deaths[eventData.recipientName] = recipientDeath
		end
		table.insert(recipientDeath.deathTimes, time())
	end
end

-----------------------------------------------------------------------------
