-----------------------------------------------------------------------------

local Roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

local AssessmentSpellColorType = Assessment:NewModule("SpellColorType", "AceEvent-2.0")
AssessmentSpellColorType.db = Assessment:AcquireDBNamespace("SpellColorType")
Assessment:RegisterDefaults("SpellColorType", "profile",
{
	spellColors =
	{
	}
})

local L = AceLibrary("AceLocale-2.2"):new("AssessmentSpellColorType")
L:RegisterTranslations("enUS", function() return {
	["Spell Colors"] = true,
	["Spell color options."] = true,
} end)

-----------------------------------------------------------------------------

local profile

-----------------------------------------------------------------------------

Assessment.ColorType.Spell = 
{
	method = function(fightData, key)
				if not profile.spellColors[key] then
					profile.spellColors[key] = { 1, 0, 0 }
					AssessmentSpellColorType:AddSpell(key)
				end
				return profile.spellColors[key]
			 end,
	description = L["Spell Colors"]
}

-----------------------------------------------------------------------------

local function spellGet(passValue)
	return unpack(AssessmentSpellColorType.db.profile.spellColors[passValue])
end

local function spellSet(passValue, r, g, b, a)
	AssessmentSpellColorType.db.profile.spellColors[passValue] = { r, g, b, a }
	AssessmentSpellColorType:TriggerEvent("Assessment:ForceDisplayUpdates")
end

-----------------------------------------------------------------------------

function AssessmentSpellColorType:OnInitialize()
	profile = self.db.profile
end

function AssessmentSpellColorType:OnEnable()
	for k,v in pairs(self.db.profile.spellColors) do
		self:AddSpell(k)
	end
end

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.spellColors =
{
	type = "group",
	name = L["Spell Colors"],
	desc = L["Spell color options."],
	order = 100,
	args = { },
}

-----------------------------------------------------------------------------

function AssessmentSpellColorType:AddSpell(spellName)
	Assessment.Options.args.data.args.spellColors.args[spellName] =
	{
		type = "color",
		name = spellName,
		desc = spellName,
		get = spellGet,
		set = spellSet,
		passValue = spellName,
		hasAlpha = true,
	}
end

-----------------------------------------------------------------------------
